from web3 import Web3
from hexbytes.main import HexBytes
# from web3._utils.datatypes import Contract

class DarkPid:

    def __init__(self,pid_hash,ark_id,externa_pid_list,externa_url_list,payload,owner) -> None:
        
        if type(pid_hash) == HexBytes:
            self.pid_hash = pid_hash
        else:
            self.pid_hash = HexBytes(pid_hash)

        self.ark = ark_id
        self.external_pid_list = externa_pid_list
        self.external_url = externa_url_list
        self.payload = payload
        self.responsible = owner

    def to_dict(self):
        """
        Converts the attributes of the class object into a dictionary.

        Returns:
            dict: A dictionary representation of the class object's attributes.
        """
        a = vars(self)
        if type(self.payload) == Payload:
        # if len(a['payload']) != 0:
            a['payload'] = self.payload.to_dict()
        return a
    
    def __is_bc_valid(bc_output):
        """
        Validates the structure and types of the provided output.

        Parameters:
            output (tuple): The output to be validated.

        Returns:
            bool: True if the output is valid, False otherwise.
        """
        if len(bc_output) != 6:
            return False

        # checa pid_hash e ark_id
        if not isinstance(bc_output[0], bytes) or not isinstance(bc_output[1], str):
            return False

        if not isinstance(bc_output[2], list) or not isinstance(bc_output[3], bytes):
            return False

        # if not isinstance(bc_output[4], bytes):
        #     return False

        return True

    def populate(dark_object,epid_db_contract,url_db_contract,payload_obj):
        assert DarkPid.__is_bc_valid(dark_object) == True, "Invalid Blockchain Output"
        # TODO: VALIDADE epid_db_contract
        # assert type(epid_db_contract) == Contract, "epid_db_contract must be web3._utils.datatypes.Contract type"

        # populate external pids
        external_pids = []
        for ext_pid in dark_object[2]:
            ext_pid = Web3.to_hex(ext_pid)
            # epid = epid_db.functions.get(ext_pid).call()
            get_func = epid_db_contract.get_function_by_signature('get(bytes32)')
            epid = get_func(ext_pid).call()

            pid_object = {'hash_id': ext_pid, 
                            'value' : epid[2], 
                            'owner:' : epid[-1]
                        }
            external_pids.append(pid_object)
        
        # deprecated

        zbytes32 = dark_object[3].hex().rstrip("0")
        if len(zbytes32) % 2 != 0:
            zbytes32 = zbytes32 + '0'
        try:    
            zbytes32 = bytes.fromhex(zbytes32).decode('utf8')
            externa_url_list = ''
        except UnicodeDecodeError:
            get_url = url_db_contract.get_function_by_signature('get(bytes32)')
            url_obj = get_url(Web3.to_hex(dark_object[3])).call()
            externa_url_list = url_obj[2]

        # if len(zbytes32) != 0:
        #     externa_url_list = zbytes32
        # else:
        #     externa_url_list = ''
        # for ext_link in dark_object[3]:
        #     externa_url_list.append(ext_link)

        pid_hash_id = Web3.to_hex(dark_object[0])
        pid_ark_id = dark_object[1]
        
        # payload = dark_object[-2].hex().rstrip("0")
        # payload_hash = dark_object[-2]

        payload = {}
        if payload_obj != None:
            payload = payload_obj
        #TODO: CRIAR O SCHEMA

        owner = dark_object[-1]
        

        return DarkPid(pid_hash_id,pid_ark_id,external_pids,externa_url_list,payload,owner)
    
    def populateDark(dark_object,epid_db_contract,url_db_contract):
        assert DarkPid.__is_bc_valid(dark_object) == True, "Invalid Blockchain Output"
        # TODO: VALIDADE epid_db_contract
        # assert type(epid_db_contract) == Contract, "epid_db_contract must be web3._utils.datatypes.Contract type"

        # populate external pids
        external_pids = []
        for ext_pid in dark_object[2]:
            ext_pid = Web3.to_hex(ext_pid)
            # epid = epid_db.functions.get(ext_pid).call()
            get_func = epid_db_contract.get_function_by_signature('get(bytes32)')
            epid = get_func(ext_pid).call()

            pid_object = {'hash_id': ext_pid, 
                            'value' : epid[2], 
                            'owner:' : epid[-1]
                        }
            external_pids.append(pid_object)
        
        # deprecated

        zbytes32 = dark_object[3].hex().rstrip("0")
        if len(zbytes32) % 2 != 0:
            zbytes32 = zbytes32 + '0'
        try:    
            zbytes32 = bytes.fromhex(zbytes32).decode('utf8')
            externa_url_list = ''
        except UnicodeDecodeError:
            get_url = url_db_contract.get_function_by_signature('get(bytes32)')
            url_obj = get_url(Web3.to_hex(dark_object[3])).call()
            externa_url_list = url_obj[2]

        # if len(zbytes32) != 0:
        #     externa_url_list = zbytes32
        # else:
        #     externa_url_list = ''
        # for ext_link in dark_object[3]:
        #     externa_url_list.append(ext_link)

        pid_hash_id = Web3.to_hex(dark_object[0])
        pid_ark_id = dark_object[1]
        
        # payload = dark_object[-2].hex().rstrip("0")
        payload_hash = dark_object[-2]
        # b'\x00' * 32 = 0
        if payload_hash != b'\x00' * 32:
            payload=''
        else:
            dp = DarkPid(pid_hash_id,pid_ark_id,external_pids,externa_url_list,'','')
            # self
            # get_payload_schema = url_db_contract.get_function_by_signature('get_payload_schema(bytes32)')
            # get_payload = url_db_contract.get_function_by_signature('get_payload(bytes32)')
            # payload_obj = get_payload(Web3.to_hex(payload_hash)).call()

            # payload_schema = get_payload_schema(Web3.to_hex(payload_obj[0])).call()

            # print(payload_schema)

            payload = 'TODO AJUSTAR'

        owner = dark_object[-1]
        

        return DarkPid(pid_hash_id,pid_ark_id,external_pids,externa_url_list,payload,owner)

        
class ExeternalPid:
    def __init__(self,id_hash,schema,value,creator) -> None:
        self.id = id_hash
        self.schema = schema
        self.pid_str_value = value
        self.creator_addr = creator

    def to_dict(self):
        """
        Converts the attributes of the class object into a dictionary.

        Returns:
            dict: A dictionary representation of the class object's attributes.
        """
        return vars(self)
    
class PayloadSchema:
    def __init__(self,schema_hash,schema_name:str,schema_version:str,configured:bool) -> None:
        self.id = schema_hash
        self.schema_name = schema_name
        self.schema_version = schema_version
        self.configured = configured

    def set_id(self, schema_hash):
        """
        Sets the schema hash ID.

        Args:
            schema_hash (str): The schema hash ID to set.
        """
        self.id = schema_hash

    def get_id(self):
        """
        Gets the schema hash ID.

        Returns:
            str: The schema hash ID.
        """
        return self.id


    def to_dict(self):
        """
        Converts the attributes of the class object into a dictionary.

        Returns:
            dict: A dictionary representation of the class object's attributes.
        """
        return vars(self)
    
    @staticmethod
    def populate(payload_schema_object):
        # assert DarkPid.__is_bc_valid(dark_object) == True, "Invalid Blockchain Output"

                
        # string schema_name;
        # string schema_version;
        # bool configured;
        schema_name = payload_schema_object[0]
        schema_version = payload_schema_object[1]
        configured = payload_schema_object[2]
        
        ps = PayloadSchema('',schema_name,schema_version,configured)

        return ps

class Payload:
    def __init__(self) -> None:
        self.payload_schema = None
        self.ips_addr = None

    def __init__(self,payload_schema:PayloadSchema,payload_addr) -> None:
        self.payload_schema = payload_schema
        self.payload_addr = payload_addr


    def to_dict(self):
        """
        Converts the attributes of the class object into a dictionary.

        Returns:
            dict: A dictionary representation of the class object's attributes.
        """
        tmp = {
            'payload_schema' : self.payload_schema.to_dict(),
            'payload_addr' : self.payload_addr
        }
        return tmp
        # return vars(self)
    
    # @staticmethod
    # def populate(dark_object, payload_schema:PayloadSchema):
    #     # assert DarkPid.__is_bc_valid(dark_object) == True, "Invalid Blockchain Output"
    #     att_value_list = dark_object[1]
    #     payload = Payload()
    #     payload.payload_schema = payload_schema
    #     payload.attributes = {}
    #     for i in range(len(att_value_list)):
    #         payload.attributes[payload_schema.attribute_list[i]] = att_value_list[i]
    #     return payload


from web3 import Web3
from hexbytes.main import HexBytes

from .gateway import DarkGateway




class DarkDecoder:

    def __init__(self, dark_gateway: DarkGateway):
        assert type(dark_gateway) == DarkGateway, "dark_gateway must be a DarkGateway object"
        self.dgw = dark_gateway

        # populate the decode dict
        decode_dict = {}
        for key in self.dgw.deployed_contracts_dict:
            c = self.dgw.deployed_contracts_dict[key]
            decode_dict[c.address] = {'contract_name': key.split('.')[0], 'contract_object': c}
        
        self.contracts_decode_dict = decode_dict

    def extract_dark_data(self,tx_addr:[HexBytes,str]):
        """
        Extracts the parameters of a dARK method invocation from a transaction hash (addres).

        Args:
            tx_addr (str): The transaction address.

        Returns:
            dict: A dictionary that contains the following information:

                * **tx_status** (str): The status of the transaction (executed or reverted).
                * **contract_name** (str): The name of the contract that was called.
                * **function** (str): The name of the function that was called.
                * **input_parameters** (dict): The parameters that were passed to the function.
                * **logs** (list): A list of dictionaries that contain the information about the events that were emitted by the contract.
            
                **Logs** detail:
                    * **contract_name** (str): The name of the contract that emitted the event.
                    * **event_name** (str): The name of the event that was emitted.
                    * **topics** (list): A list of hexadecimal strings that represent the topics of the event.
                    * **data** (str): The data that was emitted with the event.
        
        Raises:
            TypeError: If the tx_addr argument is not a HexBytes object.
        """

        # retorno contrato, funcao , parametros da uncao, status , logs
        if type(tx_addr) == HexBytes:
            _tx_addr = tx_addr
        else:
            # raise TypeError
            _tx_addr = HexBytes(Web3.toHex(tx_addr))


        _tx = self.dgw.w3.eth.get_transaction(_tx_addr)
        try:
            _cname = self.contracts_decode_dict[_tx['to']]['contract_name']
            _cobj = self.contracts_decode_dict[_tx['to']]['contract_object']
        except KeyError:
            return None
        
        c_input = _tx['input']
        cabi = _cobj.abi

        dark_service = self.dgw.deployed_contracts_dict[_cname+'.sol']
        func_obj,func_params = dark_service.decode_function_input(c_input)

        #converting func_params
        f_params = {}

        for i in func_params.keys():
            if type(func_params[i]) == bytes:
                f_payload = Web3.toHex(func_params[i])
            else:
                f_payload = func_params[i]
            f_params[i] = f_payload

        # recibo da transacao
        r_tx = self.dgw.w3.eth.get_transaction_receipt(_tx_addr)

        if r_tx['status'] != 1: # transacao falha 0-> falha; 1-> ok; 2->falha
            logs = []
        #     return _cname, func_obj, func_params, False , []
        else:
            logs = self.extract_recepits(self.contracts_decode_dict,r_tx)
        
        output = {
            'tx_status' : 'executed' if r_tx['status'] == 1 else 'reverted',
            'contract_name' : _cname,
            'function' : func_obj,
            'input_parameters' : f_params,
            'logs' :  logs
        }
        return output

    
    ###
    ### BC UTIL
    ###

    @staticmethod
    def extract_recepits(dict_deployed_contracts,r_tx):
        """
        Extracts the receipts/transaction log from the blockchain transaction.

        Args:
            dict_deployed_contracts (dict): A dictionary of deployed contracts.
            r_tx (dict): The transaction receipt.

        Returns:
            list: A list of receipts.
        """

        r_logs = []
        for tx_log in r_tx['logs']:
            r_addr = tx_log['address']
            try:
                r_cname = dict_deployed_contracts[r_addr]['contract_name']

                # primeiro topic event-signature-hash 
                # (https://besu.hyperledger.org/en/stable/public-networks/concepts/events-and-logs/#event-signature-hash)
                topics = []
                for t in tx_log['topics'][1:]:
                    topics.append(t.hex())
                
                if len(topics) > 0:
                    r_logs.append({'contract_name': r_cname, 'topics' : topics} )

            except KeyError:
                pass
            
        return r_logs


from web3 import Web3
from hexbytes.main import HexBytes

from .gateway import DarkGateway
from .util import invoke_contract_sync, invoke_contract_async
from .pid_modules import DarkPid, PayloadSchema , Payload


class DarkMap:
    
    def __init__(self, dark_gateway: DarkGateway):
        assert type(dark_gateway) == DarkGateway, "dark_gateway must be a DarkGateway object"
        assert dark_gateway.is_deployed_contract_loaded() == True, "dark_gateway must be loaded with deployed contracts"

        #dark gatewar
        self.gw = dark_gateway

        ##
        ## dARK SmartContracts
        ##

        # databases for query
        self.dpid_db = dark_gateway.deployed_contracts_dict['PidDB.sol']
        self.epid_db = dark_gateway.deployed_contracts_dict['ExternalPidDB.sol']
        self.url_db = dark_gateway.deployed_contracts_dict['UrlDB.sol']
        self.payload_schema_db = dark_gateway.deployed_contracts_dict['PayloadSchemaDB.sol']
        # authorities db to configuration
        self.auth_db = dark_gateway.deployed_contracts_dict['AuthoritiesDB.sol']
        #dARK services
        self.dpid_service = dark_gateway.deployed_contracts_dict['PIDService.sol']
        self.epid_service = dark_gateway.deployed_contracts_dict['ExternalPIDService.sol']
        self.url_service = dark_gateway.deployed_contracts_dict['UrlService.sol']
        self.auth_service = dark_gateway.deployed_contracts_dict['AuthoritiesService.sol']
        self.payload_schema_service = dark_gateway.deployed_contracts_dict['PayloadSchemaService.sol']
        #payload schema name
        # self.payload_schema_name = dark_gateway.payload_schema_name
    
    ###################################################################
    ###################################################################
    #####################  INTERNAL METHODS  #########################
    ###################################################################
    ###################################################################
    
    def __request_pid_hash(self):
        signed_tx = self.gw.signTransaction(self.dpid_service , 'assingID', self.gw.authority_addr)
        return signed_tx
    
    def __add_external_pid(self,hash_pid: HexBytes,external_pid: str,pid_shema:int):
        # 0 doi
        assert type(hash_pid) == HexBytes, "hash_pid must be a HexBytes object"
        return self.gw.signTransaction(self.dpid_service , 'addExternalPid', hash_pid, pid_shema , external_pid)
    
    def __set_url(self,hash_pid: HexBytes,ext_url: str):
        assert type(hash_pid) == HexBytes, "hash_pid must be a HexBytes object"
        return self.gw.signTransaction(self.dpid_service , 'set_url', hash_pid, ext_url)
    
    def __create_payload_schema(self,shcema_name:str,version:str,confiured:bool):
        """
            Create a payload schema and return the hash (address) of the payload
        """
        return self.gw.signTransaction(self.payload_schema_service , 
                                       'get_or_create_payload_schema',
                                       shcema_name, version, confiured)
   
    def __set_payload(self,pid_hash:HexBytes,payload_schema:HexBytes,payload_addr):
        assert type(pid_hash) == HexBytes, "pid_hash must be a HexBytes object"

        # function set_payload(bytes32 pid_hash,
        #             bytes32 payload_schema,
        #             bytes32 payload_hash)
        # print(type(pid_hash),type(payload_schema),type(payload_addr))
        # print(payload_schema)
        signed_tx = self.gw.signTransaction(self.dpid_service , 'set_payload', 
                                            pid_hash,
                                            # HexBytes(payload_schema),
                                            payload_schema,
                                            payload_addr
                                            )
        
        return signed_tx      
    

    ###################################################################
    ###################################################################
    ###################### SYNC METHODS ###############################
    ###################################################################
    ###################################################################

    def sync_request_pid_hash(self):
        """
            Request a PID and return the hash (address) of the PID
        """
        # signed_tx = self.gw.signTransaction(self.dpid_service , 'assingID', self.gw.authority_addr)
        signed_tx = self.__request_pid_hash()
        receipt, r_tx = invoke_contract_sync(self.gw,signed_tx)
        # print(receipt)
        dark_id = receipt['logs'][0]['topics'][1]
        return dark_id
    
    def bulk_request_pid_hash(self):
        """
            Request a PID and return the hash (address) of the PID
        """
        signed_tx = self.gw.signTransaction(self.dpid_service , 'bulk_assingID', self.gw.authority_addr)
        receipt, r_tx = invoke_contract_sync(self.gw,signed_tx)
        #retrieving pidhashs
        pid_hashes = []
        for i in range(len(receipt['logs'])):
            try :
                pid_hashes.append(receipt['logs'][i]['topics'][1])
                # b = dm.convert_pid_hash_to_ark(pid_hash)
            except IndexError:
                pass
        return pid_hashes
    
    def sync_request_pid(self):
        """
            Request a PID and return the ark of the PID
        """
        return self.convert_pid_hash_to_ark(self.sync_request_pid_hash())
    
    def sync_add_external_pid(self,hash_pid: HexBytes,external_pid: str,pid_schema=0):
        # assert type(hash_pid) == HexBytes, "hash_pid must be a HexBytes object"
        # signed_tx = self.gw.signTransaction(self.dpid_service , 'addExternalPid', hash_pid, 0 , external_pid)
        signed_tx = self.__add_external_pid(hash_pid,external_pid,pid_schema)
        receipt, r_tx = invoke_contract_sync(self.gw,signed_tx)
        return self.convert_pid_hash_to_ark(hash_pid)
    
    def sync_set_url(self,hash_pid: HexBytes,ext_url: str):
        # assert type(hash_pid) == HexBytes, "hash_pid must be a HexBytes object"
        # signed_tx = self.gw.signTransaction(self.dpid_service , 'set_url', hash_pid, ext_url)
        signed_tx = self.__set_url(hash_pid,ext_url)
        receipt, r_tx = invoke_contract_sync(self.gw,signed_tx)
        return self.convert_pid_hash_to_ark(hash_pid)
    
    def sync_create_payload_schema(self,shcema_name:str,version:str,confiured:bool):
        """
            Create a payload schema and return the hash (address) of the payload
        """
        signed_tx = self.__create_payload_schema(shcema_name,version,confiured)
        receipt, r_tx = invoke_contract_sync(self.gw,signed_tx)
        # return receipt['logs']
        return receipt['logs'][0]['topics'][1].hex()
    
    def sync_set_payload(self,hash_pid:HexBytes,payload_schema:HexBytes,payload_addr:str):
        # def __set_payload(self,pid_hash:HexBytes,payload_schema:HexBytes,payload_addr):
        signed_tx = self.__set_payload(hash_pid,payload_schema,payload_addr)
        receipt, r_tx = invoke_contract_sync(self.gw,signed_tx)
        return r_tx
    

        
    
    ###################################################################
    ###################################################################
    ##################### ASYNC METHODS ###############################
    ###################################################################
    ###################################################################
    
    def async_request_pid_hash(self):
        """
            Request a PID and return the hash (address) of the PID
        """
        # signed_tx = self.gw.signTransaction(self.dpid_service , 'assingID', self.gw.authority_addr)
        signed_tx = self.__request_pid_hash()
        r_tx = invoke_contract_async(self.gw,signed_tx)
        return r_tx
    
    def async_set_external_pid(self,hash_pid: HexBytes,external_pid: str,pid_schema=0):
        # assert type(hash_pid) == HexBytes, "hash_pid must be a HexBytes object"
        # signed_tx = self.gw.signTransaction(self.dpid_service , 'addExternalPid', hash_pid, 0 , external_pid)
        signed_tx = self.__add_external_pid(hash_pid,external_pid,pid_schema)
        r_tx = invoke_contract_async(self.gw,signed_tx)
        return r_tx
    
    def async_set_url(self,hash_pid: HexBytes,ext_url: str):
        # assert type(hash_pid) == HexBytes, "hash_pid must be a HexBytes object"
        # signed_tx = self.gw.signTransaction(self.dpid_service , 'set_url', hash_pid, ext_url)
        signed_tx = self.__set_url(hash_pid,ext_url)
        r_tx = invoke_contract_async(self.gw,signed_tx)
        return r_tx
    
    def async_set_payload(self,hash_pid:HexBytes,payload_schema:HexBytes,payload_addr:str):
        # def __set_payload(self,pid_hash:HexBytes,payload_schema:HexBytes,payload_addr):
        signed_tx = self.__set_payload(hash_pid,payload_schema,payload_addr)
        r_tx = invoke_contract_async(self.gw,signed_tx)
        return r_tx


    ###################################################################
    ###################################################################
    #####################  UTIL METHODS  ##############################
    ###################################################################
    ###################################################################

    def convert_pid_hash_to_ark(self,dark_pid_hash):
        """
            Convert the dark_pid_hash to a ARK identifier
        """
        return self.dpid_db.caller.get(dark_pid_hash)[1]
    
    
    
    ###################################################################
    ###################################################################
    ### Onchain core queries
    ###################################################################
    ###################################################################

    def get_pid_by_hash(self,dark_id):
        """
            Retrieves a persistent identifier (PID) by its hash value.

            Parameters:
                dark_id (str): The hash value of the PID.

            Returns:
                str: The PID associated with the given hash value.

            Raises:
                AssertionError: If the dark_id does not start with '0x'.
        """
        assert dark_id.startswith('0x'), "id is not hash"
        dark_object = self.dpid_db.caller.get(dark_id)
        # Payload
        raw_payload = dark_object[4]
        if (raw_payload[0]!= b'\x00' * 32) and (raw_payload[1]!= b'\x00' * 32):
            payload_schema = self.get_payload_schema_by_hash(raw_payload[0])
            payload_py_obj = Payload(payload_schema=payload_schema,payload_addr=raw_payload[1])
        else:
            payload_py_obj = None

        return DarkPid.populate(dark_object,self.epid_db,self.url_service,payload_py_obj)

    def get_pid_by_ark(self,dark_id):
        """
            Retrieves a persistent identifier (PID) by its ARK (Archival Resource Key) identifier.

            Parameters:
                dark_id (str): The ARK identifier of the PID.

            Returns:
                str: The PID associated with the given ARK identifier.
        """
        dark_object = self.dpid_db.caller.get_by_noid(dark_id)

        # Payload
        raw_payload = dark_object[4]
        if (raw_payload[0]!= b'\x00' * 32) and (raw_payload[1]!= b'\x00' * 32):
            payload_schema = self.get_payload_schema_by_hash(raw_payload[0])
            payload_py_obj = Payload(payload_schema=payload_schema,payload_addr=raw_payload[1])
        else:
            payload_py_obj = None

        return DarkPid.populate(dark_object,self.epid_db,self.url_service,payload_py_obj)
    
    ##
    ## PayloadSchema
    ##
    
    def get_schema_hash_id(self,schema_name:str,schema_version:str):
        """
            Retrieves the hash of a payload schema by its name and version.

            Parameters:
                schema_name (str): The name of the payload schema.
                schema_version (str): The version of the payload schema.

            Returns:
                str: The hash of the payload schema.
        """
        return '0x'+ self.payload_schema_db.caller.gen_schema_id(schema_name,schema_version).hex()
    
    def get_payload_schema_by_hash(self,ps_id:bytes):
        """
            Retrieves the payload schema by its hash.

            Parameters:
                ps_id (str): The hash of the payload schema.

            Returns:
                PayloadSchema: The payload schema object.
        """
        #entra bytes32 a conversao e feita pelo web3
        # assert dark_id.startswith('0x'), "id is not hash"
        dark_object = self.payload_schema_service.caller.get(ps_id)
        ps = PayloadSchema.populate(dark_object)
        ps.set_id(ps_id)

        return ps
    
    def get_payload_schema_by_name(self,schema_name:str,schema_version:str):
        """
            Retrieves the payload schema by its name and version.

            Parameters:
                schema_name (str): The name of the payload schema.
                schema_version (str): The version of the payload schema.

            Returns:
                PayloadSchema: The payload schema object.
        """
        
        return self.get_payload_schema_by_hash(self.get_schema_hash_id(schema_name,schema_version))
    
    ##
    ## Payload
    ##
    
    # def get_payload(self,payload_hash_id):
    #     # assert dark_id.startswith('0x'), "id is not hash"
    #     dark_object = self.dpid_db.caller.get_payload(Web3.to_hex(payload_hash_id))
    #     payload_schema_hash_id = dark_object[0]
    #     payload_schema = self.get_payload_schema_by_hash(payload_schema_hash_id)
    #     return Payload.populate(dark_object,payload_schema)
    
    # def validade_payload(self,payload: dict,payload_schema:PayloadSchema):
    #     errors = []
    #     for p in payload.keys():
    #         if p.lower() not in payload_schema.attribute_list:
    #             errors.append(p)
        
    #     if len(errors) > 0:
    #         raise Exception(" Attributes {} not in PayloadSchema {}".format(errors,payload_schema.schema_name))



